@@ -15,7 +15,8 @@ _LOGGER = logging.getLogger(__name__)
 # High: 658000c5642b6300
 def process_wohumidifier(data: bytes, mfr_data: bytes | None) -> dict[str, bool | int]:
     """Process WoHumi services data."""
-    assert mfr_data is not None
+    if mfr_data is None:
+        return {}
     _LOGGER.debug("mfr_data: %s", mfr_data.hex())
     _LOGGER.debug("data: %s", data.hex())
 
